/*
								+---------------------------------+
								|                                 |
								| *** Discrete cos. transform *** |
								|                                 |
								|  Copyright   -tHE SWINe- 2006  |
								|                                 |
								|              DCT.h              |
								|                                 |
								+---------------------------------+
*/

#ifndef __DCT_INCLUDED
#define __DCT_INCLUDED

extern const float f_pi;// = 3.14159265358979323846264338327950288419716939937510f

/*
 *	template <class T, const int n_offset>
 *	static void 1D_DCT8(T *p_dest, const T *p_src)
 *		- original code by Yann Guidon in article "Optimised Winograd DCT for the F-CPU Core 0"
 *		- T is data type operations are performed on (int16 should be sufficient)
 *		- n_offset is offset between source samples (1 for line DCT, 8 for row DCT)
 *		- m1 to m4 are precalculated constants in required data type:
 *	 		m1 = cos(4pi/16)
 *			m2 = cos(6pi/16)
 *			m3 = cos(2pi/16) - cos(6pi/16)
 *			m4 = cos(2pi/16) + cos(6pi/16)
 *			(0.707107f, 0.382683f, 0.541196f, 1.306563f)
 *		- p_dest can be equal to p_src
 *		- sadly, seems to be different type of DCT than jpeg uses
 */
template <class T, const int n_offset, const T m1, const T m2, const T m3, const T m4>
class CDCT8_1D {
public:
	void operator ()(T *p_dest, const T *p_src)
	{
		T n_tmp_7 = p_src[0 * n_offset] + p_src[7 * n_offset];
		T n_tmp_6 = p_src[1 * n_offset] + p_src[6 * n_offset];
		T n_tmp_4 = p_src[2 * n_offset] - p_src[4 * n_offset];
		T n_tmp_5 = p_src[1 * n_offset] - p_src[6 * n_offset];
		T n_tmp_9 = p_src[2 * n_offset] + p_src[5 * n_offset];
		T n_tmp_0 = p_src[3 * n_offset] + p_src[4 * n_offset];
		T n_tmp_3 = p_src[2 * n_offset] - p_src[5 * n_offset];
		T n_tmp_8 = p_src[0 * n_offset] - p_src[7 * n_offset];

		T n_tmp_1 = n_tmp_7 + n_tmp_0;
		T n_tmp_2 = n_tmp_6 - n_tmp_9;
		n_tmp_4 = n_tmp_4 + n_tmp_3;
		n_tmp_3 = n_tmp_5 + n_tmp_3;
		n_tmp_7 = n_tmp_7 - n_tmp_0;
		n_tmp_5 = n_tmp_5 + n_tmp_8;
		n_tmp_9 = n_tmp_6 + n_tmp_9;

		n_tmp_3 = m1 * n_tmp_3;
		n_tmp_2 = n_tmp_2 + n_tmp_7;
		n_tmp_6 = n_tmp_4 - n_tmp_5;

		n_tmp_4 = m3 * n_tmp_4;
		n_tmp_5 = m4 * n_tmp_5;
		n_tmp_2 = m1 * n_tmp_2;
		n_tmp_6 = m2 * n_tmp_6;

		p_dest[0 * n_offset] = n_tmp_1 + n_tmp_9;
		p_dest[4 * n_offset] = n_tmp_1 - n_tmp_9;

		n_tmp_1 = n_tmp_3 + n_tmp_8;
		n_tmp_3 = n_tmp_8 - n_tmp_3;
		p_dest[2 * n_offset] = n_tmp_7 + n_tmp_2;
		n_tmp_4 = n_tmp_4 + n_tmp_6;
		p_dest[6 * n_offset] = n_tmp_7 - n_tmp_2;
		n_tmp_2 = n_tmp_5 + n_tmp_6;

		p_dest[3 * n_offset] = n_tmp_3 - n_tmp_4;
		p_dest[5 * n_offset] = n_tmp_3 + n_tmp_4;
		p_dest[1 * n_offset] = n_tmp_1 + n_tmp_2;
		p_dest[7 * n_offset] = n_tmp_1 - n_tmp_2;
	}
};

class CReferenceDCT8 {
public:
	static void Forward(float *p_dest, const float *p_src);
	static void Inverse(float *p_dest, const float *p_src);
};

class CFastDCT8 {
protected:
	static const float m_p_prescale_table[8];

public:
	static const float *p_PrescaleTable();
	static void Inverse(float *p_dest, const float *p_src);
};

#endif //__DCT_INCLUDED
