/*
								+----------------------------------+
								|                                  |
								|   ***  3D modelling tools  ***   |
								|                                  |
								|   Copyright  -tHE SWINe- 2005   |
								|                                  |
								|            Geometry.h            |
								|                                  |
								+----------------------------------+
*/

#ifndef __GEOMETRY_INCLUDED
#define __GEOMETRY_INCLUDED

template <class T> class tiny_vector {
protected:
	T *m_p_list;
public:
	tiny_vector() :m_p_list(0) {}
	~tiny_vector() { clear(); }

	void clear()
	{
		if(m_p_list) {
			delete[] m_p_list;
			m_p_list = 0;
		}
	}

	inline int empty() const
	{
		return m_p_list == 0;
	}

	int size(const T &r_terminator) const
	{
		if(!m_p_list)
			return 0;
		int n_size = 0;
		while(m_p_list[n_size] != r_terminator)
			n_size ++;
		return n_size;
	}

	inline T operator [](int n_index) const
	{
		return m_p_list[n_index];
	}

	inline T &operator [](int n_index)
	{
		return m_p_list[n_index];
	}

	inline T *begin()
	{
		return m_p_list;
	}

	inline const T *begin() const
	{
		return m_p_list;
	}

	inline T *end(const T &r_terminator)
	{
		return m_p_list + size(r_terminator);
	}

	inline const T *end(const T &r_terminator) const
	{
		return m_p_list + size(r_terminator);
	}

	inline T *rbegin(const T &r_terminator)
	{
		return m_p_list + size(r_terminator) - 1;
	}

	inline const T *rbegin(const T &r_terminator) const
	{
		return m_p_list + size(r_terminator) - 1;
	}

	inline T *rend()
	{
		return m_p_list - 1;
	}

	inline const T *rend() const
	{
		return m_p_list - 1;
	}

	void insert(T *p_after, const T &r_elem, const T &r_terminator)
	{
		int n_size = size(r_terminator);

		T *p_list = new T[n_size + 2];

		if(!p_list) {
			clear();
			return;
		}

		for(T *p_list_ptr = p_list, *p_list_src = m_p_list; p_list_src < p_after + 1;)
			*p_list_ptr ++ = *p_list_src ++;
		for(T *p_list_ptr = p_list + (p_after - m_p_list + 2), *p_list_src = p_after + 1,
		   *p_list_end = p_list + (n_size + 1); p_list_ptr < p_list_end;)
			*p_list_ptr ++ = *p_list_src ++;
		// copy, make space for a new one

		p_list[p_after + 1 - m_p_list] = r_elem;
		p_list[n_size + 1] = r_terminator;
		// insert a new one and terminate

		clear();

		m_p_list = p_list;
	}

	void insert(T *p_after, const T *p_elem_begin, const T *p_elem_end, const T &r_terminator)
	{
		int n_size = size(r_terminator);

		T *p_list = new T[n_size + 1 + (p_elem_end - p_elem_begin)];

		if(!p_list) {
			clear();
			return;
		}

		for(T *p_list_ptr = p_list, *p_list_src = m_p_list; p_list_src < p_after + 1;)
			*p_list_ptr ++ = *p_list_src ++;
		for(T *p_list_ptr = p_list + (p_after - m_p_list + 1 + (p_elem_end - p_elem_begin)), *p_list_src = p_after + 1,
		   *p_list_end = p_list + (n_size + (p_elem_end - p_elem_begin)); p_list_ptr < p_list_end;)
			*p_list_ptr ++ = *p_list_src ++;
		// copy, make space for a new one

		p_list[n_size + (p_elem_end - p_elem_begin)] = r_terminator;
		for(T *p_list_ptr = p_list + (p_after + 1 - m_p_list); p_elem_begin < p_elem_end;)
			*p_list_ptr ++ = *p_elem_begin ++;
		// insert a new one and terminate

		clear();

		m_p_list = p_list;
	}

	inline void push_back(const T &r_elem, const T &r_terminator)
	{
		insert(end(r_terminator) - 1, r_elem, r_terminator);
	}

	inline void push_back(const T *p_elem_begin, const T *p_elem_end, const T &r_terminator)
	{
		insert(end(r_terminator) - 1, p_elem_begin, p_elem_end, r_terminator);
	}

	inline void push_front(const T &r_elem, const T &r_terminator)
	{
		insert(m_p_list - 1, r_elem, r_terminator);
	}

	inline void push_front(const T *p_elem_begin, const T *p_elem_end, const T &r_terminator)
	{
		insert(m_p_list - 1, p_elem_begin, p_elem_end, r_terminator);
	}

	void erase(T *p_elem, const T &r_terminator)
	{
		int n_size = size(r_terminator);

		if(!(n_size - 1)) {
			clear();
			return;
		}

		T *p_list = new T[n_size];

		if(!p_list) {
			clear();
			return;
		}

		for(T *p_list_ptr = p_list, *p_src_list = m_p_list; p_src_list < p_elem;)
			*p_list_ptr ++ = *p_src_list ++;
		for(T *p_list_ptr = p_list + (p_elem - m_p_list), *p_src_list = p_elem + 1,
		   *p_list_end = p_list + (n_size - 1); p_list_ptr < p_list_end;)
			*p_list_ptr ++ = *p_src_list ++;
		// copy, leave space took up by erased

		p_list[n_size - 1] = r_terminator;
		// terminate

		clear();

		m_p_list = p_list;
	}

	inline void erase_back(const T &r_terminator)
	{
		erase(rbegin(r_terminator), r_terminator);
	}

	inline void erase_front(const T &r_terminator)
	{
		erase(m_p_list, r_terminator);
	}

	void erase(T *p_elem_begin, T *p_elem_end, const T &r_terminator)
	{
		int n_size = size(r_terminator);

		if(!(n_size - (p_elem_end - p_elem_begin))) {
			clear();
			return;
		}

		T *p_list = new T[n_size + 1 - (p_elem_end - p_elem_begin)];

		if(!p_list) {
			clear();
			return;
		}

		for(T *p_list_ptr = p_list, *p_src_list = m_p_list; p_src_list < p_elem_begin;)
			*p_list_ptr ++ = *p_src_list ++;
		for(T *p_list_ptr = p_list + (p_elem_begin - m_p_list), *p_src_list = p_elem_end,
		   *p_list_end = p_list + (n_size - (p_elem_end - p_elem_begin)); p_list_ptr < p_list_end;)
			*p_list_ptr ++ = *p_src_list ++;
		// copy, leave space took up by erased

		p_list[n_size - (p_elem_end - p_elem_begin)] = r_terminator;
		// terminate

		clear();

		m_p_list = p_list;
	}
};

class CVolumeData {
public:
	virtual int n_Width() const = 0;
	virtual int n_Height() const = 0;
	virtual int n_Depth() const = 0;

	virtual bool PutPixel(int n_x, int n_y, int n_z) = 0;
};

enum {
	rp_Lines,		// lines between two vertices
	rp_Rays,		// rays, beginning in first vertex and going trough second vertex into infinity
					// (pixels are drawn in ray direction order, ie. near first)
	rp_Triangles,	// triangles (cells only touched by triangle are filled as well)
	rp_Quads,		// quadriliterals (cells only touched by quad are filled as well)
	rp_Polygon		// a single polygon (cells only touched by poly are filled as well)
};
// rasterizer primitives

class CVolumeRasterizer {
public:
	CVolumeRasterizer();
	~CVolumeRasterizer();

	void SetTransform(Vector3f v_bound_min, Vector3f v_bound_max);
	// set such a transformation matrix so objects inside given
	// bounding box are mapped to the whole volume

	bool Begin(int n_primitive_type, CVolumeData &r_t_output);
	bool Vertex3f(const Vector3f &r_v_vertex);
	bool End();
};
// volume rasterizer, used for simple spatial partitioning and collision
// detection (for example ray / poly collisions)
// note quite a big memory requirements, volume of:
// 8x8x8 = 512 boxes (x4 = 1 kB)
// 16x16x16 = 4k boxes (x4 = 16 kB)
// 32x32x32 = 32k boxes (x4 = 128 kB)
// 64x64x64 = 256k boxes (x4 = 1 MB)
// 128x128x128 = 2M boxes (x4 = 8 MB)
// 256x256x256 = 16M boxes (x4 = 64 MB)
// 512x512x512 = 128M boxes (x4 = 512 MB)
// when using std::vector list template, it's 3x4 bytes for pointers (front, back, end), i.e.
// 12x more it could've been better to use 4 bytes for pointer only and keep lists with
// terminators that is 4x more + terminators = count of non-empty cells * size of data in list
// (note it will be propably slower)

enum {
	max_Vertex_Texcoords = 4,
	max_Surface_Textures = 4
};
// number of texcoords per-vertex
// and number of per-surface texture id's per poly

struct TVertex /*: public TVertexStruct*/ {
	int n_flag; // must be integer-type to be able to hold indices/id-s; it has no long-term purpose, it's temp variable, needed during some operations only
	Vector3f v_pos;
	Vector3f v_normal;
	Vector4f v_texcoord[max_Vertex_Texcoords];
	Vector4f v_color;

	inline Vector3f v_Pos() const
	{
		return v_pos;
	}

	inline Vector3f &v_Pos()
	{
		return v_pos;
	}

	TVertex Lerp(float t, const TVertex &r_right) const
	{
		TVertex t_tmp;

		t_tmp.v_Pos() = v_Pos() + (r_right.v_Pos() - v_Pos()) * t;
		// v_pos is !not! lerped in TVertexStruct::Lerp (because of function work with
		// different datatypes (TVertex instead of TVertexStruct)) - this is good, because
		// we save one function call
		t_tmp.v_normal = v_normal + (r_right.v_normal - v_normal) * t;
		for(int i = 0; i < max_Vertex_Texcoords; i ++)
			t_tmp.v_texcoord[i] = v_texcoord[i] + (r_right.v_texcoord[i] - v_texcoord[i]) * t;
		t_tmp.v_color = v_color + (r_right.v_color - v_color) * t;

		return t_tmp;
	}

	bool operator ==(const TVertex &r_other) const
	{
		return b_Compare(r_other, f_epsilon);
	}

	bool b_Compare(const TVertex &r_other, float f_epsilon_ex = f_epsilon) const
	{
		if((r_other.v_color - v_color).f_Length() > f_epsilon_ex ||
		   (r_other.v_normal - v_normal).f_Length() > f_epsilon_ex ||
		   (r_other.v_pos - v_pos).f_Length() > f_epsilon_ex)
			return false;

		for(int i = 0; i < max_Vertex_Texcoords; i ++) {
		   if((r_other.v_texcoord[i] - v_texcoord[i]).f_Length() > f_epsilon_ex)
			   return false;
		}

		return true;
	}

	/*
	 *	void Clear_ExtraFields()
	 *		- fill vertex fields, Create_ functions doesn't fill with some default values
	 *		  to avoid grabage in those
	 *		- namely clears flag to zero, color to (0, 0, 0, 1) and
	 *		  texcoords 0 .. max_Vertex_Texcoords to (0, 0, 0, 1)
	 */
	void Clear_ExtraFields()
	{
		n_flag = 0;
		v_color = Color4f(0, 0, 0, 1);
		for(Vector4f *p_texcoord = v_texcoord, *p_end = v_texcoord + max_Vertex_Texcoords;
		   p_texcoord < p_end;)
			*p_texcoord ++ = Vector4f(0, 0, 0, 1);
	}
};

class CGeometry;

struct TRefVertex /*: TVertexStruct*/ {
	class CCantInsertVertex_Exception {};
	// thrown by TRefVertex Lerp(float t, const TRefVertex &r_right) const

	TVertex *p_vertex;
	CGeometry *p_geom;

	inline Vector3f v_Pos() const
	{
		return p_vertex->v_Pos();
	}

	inline Vector3f &v_Pos()
	{
		return p_vertex->v_Pos();
	}

	TRefVertex Lerp(float t, const TRefVertex &r_right) const; // throw CCantInsertVertex_Exception
};

class CRefPolygon : public CPolygon<TRefVertex> {
friend class CGeometry;
public:
	inline const TVertex t_Vertex(int n_index) const
	{
		return *m_vertex_list[n_index].p_vertex;
	}

	inline TVertex *&r_p_Vertex(int n_index)
	{
		return m_vertex_list[n_index].p_vertex;
	}
	// want to bypass access to TRefVertex structure

	inline bool operator =(const CRefPolygon &r_poly)
	{
		return *(CPolygon<TRefVertex>*)this = (CPolygon<TRefVertex>)r_poly;
	}

private:
	void SetVertexGeometryPointer(int n_index, CGeometry *p_geometry) // should be called by CGeometry only (it tells polygon where to store new vertices, derived from this one)
	{
		m_vertex_list[n_index].p_geom = p_geometry;
	}
};

enum {
	surf_Solid = 1,
	surf_Transparent = 2,
	surf_NoDraw = 4,
	surf_NoShadows = 8,
	surf_NoDecal = 16,
	surf_Detail = 32
};

struct TPolygon {
	CRefPolygon *p_poly;
	std::vector<int> **p_neighbour_index_list;
	int n_neighbour_edge_num;
	// there's as much neighbour lists as polygon edges (ie. there can be >1
	// neighbour on single edge) it's not generated automatically, but when
	// it's found and you modify poygons (using CGeometry modelling functions),
	// it's automatically updated
	int n_material, n_surface_map[max_Surface_Textures];
	// material is basic material (texture), n_surface_map is designed
	// to hold per-surface texture id such as of lightmap or normalmap
	int n_surface; // smoothing purposes (polygons with the same n_surface
	// will be smoothed together)
	int n_surface_flags; // BSP/vis/whatever map tool purposes
};

struct TTriStrip {
	std::vector<TVertex*> m_vertex_list;
	int n_material;
};

#if defined(_MSC_VER) && !defined(__MWERKS__) // msvc
#pragma warning(4:4786)
// switch-off warning 4786 ("too long class name truncated to 255 chars in debug info")
#endif

enum ECoordMethod {
	coord_XYZ_to_STR,
	coord_Planar,
	coord_Box,
	coord_Cylinder,
	coord_Sphere,
	coord_Tangent_of_1,
	coord_Tangent_of_2,
	coord_Tangent_of_3,
	coord_Tangent_of_4,
	coord_BiTangent_of_1 = coord_Tangent_of_1 + max_Vertex_Texcoords,
	coord_BiTangent_of_2,
	coord_BiTangent_of_3,
	coord_BiTangent_of_4
};

enum ESurfMethod {
	surf_Planar,
	surf_Unwrap
};

/*class CVertexHash {
protected:
	std::vector<TVertex*> **m_p_vertex_hash_table;
	float m_f_vertex_hash_table_cell_size;
	Vector3f m_v_bounding_box[2]; // 0 = min, 1 = max
	const float m_f_vertex_cell_overlap_ratio;
	const float m_f_min_vertex_cell_overlap;
	const int m_n_vertex_hash_table_size;
	bool m_b_bounding_box_dirty_flag;
	bool m_b_vertex_hash_dirty_flag;

public:
	CVertexHash(int n_vertex_hash_table_size = 16,
		float f_vertex_cell_overlap_ratio = .1f, float f_min_vertex_cell_overlap = f_epsilon);
	~CVertexHash();

	inline bool b_Dirty() { return m_b_vertex_hash_dirty_flag || m_b_bounding_box_dirty_flag; }

	void SetBoundingBoxDirtyFlag();
	void SetVertexHashDirtyFlag();

	bool AddVertex_Lazy(TVertex *p_new_vertex);
	bool RemoveVertex(TVertex *p_vertex);

	bool Update();

	class filter_iterator {
	protected:
		std::vector<TVertex*>::iterator m_p_value;

	public:
		filter_iterator(float f_epsilon, Vector3f v_pos);
	};

	int n_HashFunction(Vector3f v_pos);
	const std::vector<TVertex*> *p_NearVertices(int n_hash);
};*/

class CGeometry {
protected:
	std::vector<TVertex*> m_vertex_list;
	std::vector<TPolygon> m_poly_list;

	static int m_n_surface_space; // surface id's (each surface can be more polygons, it's generally part of geometric primitive, such as bow wall or cylinder cap. each surface has smooth vertex normals for all polygons within)

	bool m_b_bounding_box_dirty_flag;
	bool m_b_poly_hash_dirty_flag;
	bool m_b_vertex_hash_dirty_flag;
	bool m_b_vertex_normal_smooth_group_dirty_flag;
	bool m_b_vertex_normal_smooth_all_dirty_flag;
	bool m_b_poly_neighbours_dirty_flag;

	const int m_n_polys_per_hash_record;
	const float m_f_vertex_cell_overlap_ratio;
	const float m_f_min_vertex_cell_overlap;
	const int m_n_vertex_hash_table_size;

	std::vector<TVertex*> **m_p_vertex_hash_table;
	float m_f_vertex_hash_table_cell_size; // you tell it from bounding box and m_n_vertex_hash_table_size

	std::vector<int> **m_p_poly_hash_table;
	int m_n_poly_hash_table_size; // you tell from current polygon count and m_n_polys_per_hash_record

	Vector3f m_v_bounding_box[2]; // 0 = min, 1 = max

public:
	CGeometry(int n_polys_per_hash_record = 32, int n_vertex_hash_table_size = 16,
		float f_vertex_cell_overlap_ratio = .1f, float f_min_vertex_cell_overlap = f_epsilon);
	~CGeometry();

	void Delete();
	// delete contents of whole geometry

	Vector3f v_Bound_Min();
	Vector3f v_Bound_Max();
	// min, max

	void UpdateBoundingBox();
	bool UpdateVertexHash();
	bool UpdatePolygonHash();
	bool UpdatePolygonNeighbours();
	bool UpdateVertexNormals_SmoothAll();
	bool UpdateVertexNormals_SmoothGroups();

	inline int n_Vertex_Num() const
	{
		return m_vertex_list.size();
	}

	inline int n_Polygon_Num() const
	{
		return m_poly_list.size();
	}

	inline const TVertex t_Vertex(int n_index) const
	{
		return *m_vertex_list[n_index];
	}

	inline CRefPolygon t_Polygon(int n_index) const
	{
		return *m_poly_list[n_index].p_poly;
	}

	inline int n_PolygonMaterial(int n_index) const
	{
		return m_poly_list[n_index].n_material;
	}

	inline int &n_PolygonMaterial(int n_index)
	{
		return m_poly_list[n_index].n_material;
	}

	inline int n_PolygonSurfaceMap(int n_index, int n_map_index) const
	{
		return m_poly_list[n_index].n_surface_map[n_map_index];
	}

	inline int &n_PolygonSurfaceMap(int n_index, int n_map_index)
	{
		return m_poly_list[n_index].n_surface_map[n_map_index];
	}

	inline int n_PolygonSurfaceFlags(int n_index) const
	{
		return m_poly_list[n_index].n_surface_flags;
	}

	inline int &n_PolygonSurfaceFlags(int n_index)
	{
		return m_poly_list[n_index].n_surface_flags;
	}

	inline TVertex &r_Vertex(int n_index)
	{
		m_b_bounding_box_dirty_flag = true;
		m_b_vertex_hash_dirty_flag = true;
		m_b_vertex_normal_smooth_group_dirty_flag = true;
		m_b_vertex_normal_smooth_all_dirty_flag = true;
		m_b_poly_neighbours_dirty_flag = true;

		return *m_vertex_list[n_index];
	}

	inline CRefPolygon &r_Polygon(int n_index)
	{
		m_b_bounding_box_dirty_flag = true;
		m_b_poly_hash_dirty_flag = true;
		m_b_vertex_hash_dirty_flag = true;
		m_b_vertex_normal_smooth_group_dirty_flag = true;
		m_b_vertex_normal_smooth_all_dirty_flag = true;
		m_b_poly_neighbours_dirty_flag = true;

		return *m_poly_list[n_index].p_poly;
	} // don't know why, but all inlines have to be in header otherwise i get linker error

	const std::vector<int> *p_PolygonNeighbours(int n_index, int n_edge);
	//const std::vector<int> *p_PolygonNeighbours(int n_index, int n_edge) const;
	// geometry access tools

	void DeletePolygons(int n_index, int n_count = 1);
	bool DeleteVertices(int n_index, int n_count = 1);
	// delete polygons

	bool AddVertices(const void *p_vertex, int n_vertex_num, int n_stride,
		int n_position_offset = 0, int n_texcoord_offset = -1, int n_texcoord_num = 2,
		GLenum n_texcoord_type = GL_FLOAT, int n_normal_offset = -1, int n_color_offset = -1,
		GLenum n_color_format = GL_RGB, GLenum n_color_type = GL_UNSIGNED_BYTE,
		bool b_2d_vertices = false);
	// add vertices, the new vertices are added to back of m_vertex_list
	// n_*_offset is component offset in vertex structure (-1 = component not present;
	// positions must be always present)
	// positions and normals are always 3-component floats
	// n_texcoord_num is number of texture coordinates, it can be number
	// from 0 trough 4 * max_Vertex_Texcoords
	// n_color_format is format of color, supported formats are: GL_RED, GL_GREEN, GL_BLUE,
	// GL_ALPHA, GL_RGB, GL_RGBA, GL_BGR, GL_BGRA, GL_LUMINANCE, GL_LUMINANCE_ALPHA, GL_INTENSITY
	// color components that aren't specified are set to 0, unspecified alpha channel is set to 1

	bool AddGLVertices(const void *p_vertex, int n_vertex_num, GLenum n_vertex_format);
	// n_vertex_format is one of GL_V2F, GL_V3F, GL_C4UB_V2F, GL_C4UB_V3F, GL_C3F_V3F,
	// GL_N3F_V3F, GL_C4F_N3F_V3F, GL_T2F_V3F, GL_T4F_V4F, GL_T2F_C4UB_V3F, GL_T2F_C3F_V3F,
	// GL_T2F_N3F_V3F, GL_T2F_C4F_N3F_V3F, GL_T4F_C4F_N3F_V4F

	bool GetVertices(void *&r_p_vertex, int &r_n_length, int n_stride, int n_position_offset = 0,
		int n_texcoord_offset = -1, int n_texcoord_num = 2, GLenum n_texcoord_type = GL_FLOAT,
		int n_normal_offset = -1, int n_color_offset = -1, GLenum n_color_format = GL_RGB,
		GLenum n_color_type = GL_UNSIGNED_BYTE, int n_vertex_coord_dimension = 3);
	// get vertices in required custom format
	// pass vertex buffer pointer, if size is sufficient do nothing, otherwise (re)alloc

	bool GetGLVertices(void *&r_p_vertex, int &r_n_length, GLenum n_vertex_format);
	// get vertices in required open-gl format
	// pass vertex buffer pointer, if size is sufficient do nothing, otherwise (re)alloc

	bool AddPrimitives(GLenum n_prim_type, int n_vertex_offset,
		const int *p_vertex_index_list, int n_vertex_num, int n_material,
		bool b_smooth, bool b_smooth_prev);
	// n_prim_type can be one of GL_TRIANGLES, GL_QUADS, GL_TRIANGLE_FAN, GL_TRIANGLE_STRIP,
	// GL_QUAD_STRIP, GL_POLYGON

	TVertex *p_AddVertex(TVertex t_vertex);
	TRefVertex t_AddRefVertex(TVertex t_vertex); // in case there's not enough memory, TRefVertex::p_vertex will be 0
	// for polygons in case they need to add vertex

	bool Create_Box(Vector3f v_org, Vector3f v_x, Vector3f v_y, Vector3f v_z, int n_material, int n_tess_x = 1, int n_tess_y = 1, int n_tess_z = 1);
	bool Create_Grid(Vector3f v_org, Vector3f v_x, Vector3f v_z, int n_material, int n_tess_x = 16, int n_tess_y = 16);
	bool Create_Heightfield(Vector3f v_org, Vector3f v_x, Vector3f v_y, Vector3f v_z, int n_material, int n_tess_x, int n_tess_y, unsigned char *p_heightmap_grayscale);
	bool Create_Cylinder(Vector3f v_org, Vector3f v_x, Vector3f v_y, Vector3f v_z, int n_material, int n_cap_material, int n_tess_r = 12, int n_tess_a = 1, int n_tess_cap = 1);
	bool Create_Tube(Vector3f v_org, Vector3f v_x, Vector3f v_y, Vector3f v_z, float f_inner_r, int n_material, int n_cap_material, int n_tess_r = 12, int n_tess_a = 1);
	bool Create_Cone(Vector3f v_org, Vector3f v_x, Vector3f v_y, Vector3f v_z, int n_material, int n_cap_material, int n_tess_r = 12, int n_tess_a = 1, int n_tess_cap = 1);
	bool Create_Sphere(Vector3f v_org, Vector3f v_x, Vector3f v_y, Vector3f v_z, int n_material, int n_tess_r = 24, int n_tess_a = 12);
	bool Create_Hemisphere(Vector3f v_org, Vector3f v_x, Vector3f v_y, Vector3f v_z, int n_material, int n_cap_material, int n_tess_r = 24, int n_tess_a = 6, int n_tess_cap = 1);
	bool Create_Torus(Vector3f v_org, Vector3f v_x, Vector3f v_y, Vector3f v_z, int n_material, float f_inner_r, int n_tess_r = 30, int n_tess_sub_r = 12);
	bool Create_Octagon(Vector3f v_org, Vector3f v_x, Vector3f v_y, Vector3f v_z, int n_material);
	bool Create_Tetragon(Vector3f v_org, Vector3f v_x, Vector3f v_y, Vector3f v_z, int n_material);
	// create custom size, custom tesselation geometric primitives

	bool AddCapVertices(Vector3f v_org, Vector3f v_x, Vector3f v_y, Vector3f v_z, int n_tess_r, int n_tess_cap, bool b_flip_normals);
	bool AddCylVertices(Vector3f v_org, Vector3f v_x, Vector3f v_y, Vector3f v_z, int n_tess_r, int n_tess_a, bool b_flip_normals); // only coat vertices
	bool AddGridQuads(int n_first_vertex, int n_tess_x, int n_tess_y, bool b_flip, int n_material);
	bool AddCapPolygons(int n_first_vertex, int n_tess_r, int n_tess_cap, bool b_flip, int n_material);
	// helper functions for creating basic parts of primitives (cap is triangulated circle, cyl is cylinder without caps, grid is grid of quads)
	// created surfaces are smooth (single surface-id for all polygons generated in single call to Add*)

	std::vector<std::vector<TVertex*>*> *p_VertexMergeTable(bool b_compare_position_only, float f_epsilon_ex = f_epsilon); // if you don't give pointer to a valid hash table, it generates one
	// table of vertices, sharing the same position (and according to b_compare_position_only also other parameters)

	bool CalcVertexNormals(bool b_ignore_groups); // t_odo - maybe call some function that would separate surfaces first to avoid using single vertex by >1 surfaces
	// calc vertex normals

	bool FindNeighbours();
	// find polygon neighbours

	bool Remove_DegenerateEdges(float f_edge_epsilon_ex, bool b_weld_vertices);
	// remove all edges, shorter than f_edge_epsilon_ex, if b_weld_vertices is true, move
	// vertices within f_edge_epsilon_ex to a single position (position of one of vertices, not avg)

	bool Remove_TJunctions(float f_epsilon_ex, float f_edge_epsilon_ex,
		bool b_weld_vertices, bool b_stick_vertices_to_edge);
	// remove T-junctions (T-junction is edge of one polygon, lying on the edge of other poly,
	// but their end vertices aren't on the same distance along the edge, yielding thin gap
	// artifact when rasterized) f_epsilon_ex is max distance of vertex from edge to be considered
	// on-edge, edges shorter than f_edge_epsilon_ex are ignored
	// if b_weld_vertices is true, vertices within b_weld_vertices will be moved to single
	// position; if b_stick_vertices_to_edge is true, vertices will be moved towards edge axis,
	// it might eliminate some rasterizer artifacts later on

	bool Break_SubMeshVertices(const int *p_polygon_indices, int n_polygon_num);
	// duplicate vertices in such a manner that selected polygons are
	// separated from the rest of geometry

	// todo handle p_polygon_indices == 0 as first n_polygon_num polys; do it with all functions here
	bool Generate_Texcoords(ECoordMethod n_method,
		const int *p_polygon_indices, int n_polygon_num,
		int n_tex_channel, int n_s_coord, int n_t_coord, int n_r_coord, int n_w_coord,
		const Matrix4f &m_world_transform, Vector3f v_scale);
	// generate texture coordinates for selected polygons, to some texture stage coords,
	// with some orientation and scaling (interpretation depends on used method)

	bool Alloc_Surfaces(ESurfMethod n_method, const int *p_polygon_indices, int n_polygon_num,
		float f_max_deviation, Vector3f v_scale, int n_max_texture_size, int n_border_width,
		int n_surface_channel, int n_tex_channel, int n_s_coord, int n_t_coord); // todo
	// generate texture coordinates and per-surface texture id-s for selected poylgons
	// there is some scale, maximal textutre size (big polygons are automatically subdivided)
	// border width (space necessary to be left between non-connected triangles) and maximal
	// error bound; this just generates surfaces with different id's, containing part or whole
	// connected mesh with size up to max texture size; now it's necessary to force them
	// into some texture pages (and since i want to do texture caching realtime, it's no longer
	// bussiness of CGeometry)

	bool MakeTris();
	bool Tesselate(float f_normal_tension);
	// tesselation functions

	bool b_IsEnclosed_Euler(bool &r_b_enclosed) /*const - can't be const since it may require recomputation of certain lookups*/;
	bool b_IsEnclosed_Adjacency(bool &r_b_enclosed) /*const - can't be const since it may require recomputation of certain lookups*/;
	bool Cut(const Plane3f &r_t_plane, EPlanePos n_desired_half, float f_epsilon_ex = f_epsilon, CGeometry *p_other_half = 0);
	bool Slice(const Plane3f &r_t_plane, const int *p_polygon_indices, int n_polygon_num, float f_epsilon_ex = f_epsilon);
	// splitting functions

	bool FindColliders(std::vector<int> &r_poly_indices_a,
		std::vector<int> &r_poly_indices_b, std::vector<Vector2i> &r_collider_list);
	// find polygons from list a that collide with those from list b

	bool Extrude(const int *p_polygon_indices, int n_polygon_num, float f_amount, bool b_smooth, bool b_smooth_sides);
	int Hollow(float f_hollow_size); // todo - hollow geometry (may have some undesirable effects on bad-formed geometry)
	bool Boolean(const CGeometry &r_geometry, bool b_mesh_a, bool b_mesh_b,
		bool b_mesh_a_inside_b, bool b_mesh_b_inside_a, bool b_flip_a, bool b_flip_b);
	// some modelling functions

	bool Insert_EdgeQuads(int n_tex_channel, bool b_flip, int n_material);
	// insert (degenerate) quads under every mesh edge (can be used for example for GPU
	// stencil volume extrusion or for rendering silhouettes)
	// it also stores face plane (plane ~ [a, b, c, d]) into texture channel n_tex_channel
	// of every vertex (which is essential for GPU to know if face is visible and to
	// accordingly offset or not offset vertex)

	std::vector<int> *p_Outline_IndexList(const int *p_polygon_indices, int n_polygon_num);
	// create index of vertices, needed to connect to create (nonconvex) polygon, covering all listed polygons
	// if polygons form more separate surfaces or surface with holes, it may contain double edges

	bool OptimizeVertices(float f_epsilon_ex = f_epsilon); // merge vertices with the same position & data; call when you're done
	bool OptimizeTris(const int *p_polygon_indices, int n_polygon_num, float f_epsilon_ex); // optimize tris (can be polygon or tris, edge contraction -> result is mostly tris)
	bool OptimizeTrisLossy(int n_target_poly_count); // compute certain lod
	bool Optimize(const int *p_polygon_indices, int n_polygon_num, float f_epsilon_ex); // connect tris to polygons (good to be called after OptimizeTris if you wish to render polygons. otherwise you'd propadly call MakeTris() and p_TriStripify())
	//
	std::vector<TTriStrip> *p_TriStripify(const int *p_poly_list, int n_poly_num); // create triangle strips
	// mesh optimizing functions - todo

	unsigned int **p_IndexList(bool b_single_memblock = true);
	// return list of arrays of vertex indices
	// (each array has the same length as number
	// of vertices of polygon with the same index)

	bool operator =(const CGeometry &r_geometry);
	bool operator +=(const CGeometry &r_geometry);
	// copying functions

protected:
	//void ClearUnusedVertexFields(TVertex *p_vertex); moved to TVertex::Clear_ExtraFields()

	static int n_HashFunction(Vector3f v_pos, int n_table_size, float f_cell_size);
	std::vector<TVertex*> **p_HashVertices(int n_table_size = 32, float f_cell_size = 10.0f, float f_cell_overlap = .1f) const; // when table size is 32, table takes 32 * 32 * 32 * sizeof(TVertex*) = 128kB of memory
	static int n_HashFunction(const TVertex *p_vertex_address, int n_table_size);
	std::vector<int> **p_HashPolygons(int n_table_size) const; // create table to speed-up neighbour searches, etc ...
	// hashing functions to speed-up processing

	bool Alloc_PolygonNeighbourLists();

	bool _Sort_PolyList(const Plane3f &r_plane, std::vector<int> &r_onplane_list,
		std::vector<int> &r_list, int &r_n_lo, int &r_n_hi, int &r_n_duplicated,
		std::vector<Vector2i> &r_collider_list, bool b_fill_remove_onplane, bool b_fill_collider);
	// polygon sorting function
	// it resembles quicksort a bit, it's used by FindColliders()

	bool Find_MinPolygonAlignedBoundingBoxBasis(const CRefPolygon *p_poly,
		Vector3f &r_v_x, Vector3f &r_v_y, Vector3f &r_v_z);
	// used in AllocSurfaces()

	void ContractEdge_UpdateNeighbours(TPolygon &r_t_polygon, int n_edge, float f_position,
		const std::vector<std::vector<TVertex*>*> *p_merge_table);
	// contract polygon edge of r_t_polygon and of all it's neighbours, update polygon
	// neighbour lists (if m_b_polygon_neighbours_dirty_flag == false).
	// final edge vertex will be between first edge vertex (as sorted in r_t_polygon,
	// (f_position == .0), and second edge vertex (f_position == 1.0)
	// each vertex's flag has to contain index of merge list, containing it

	/*
	 *	TVertex CGeometry::t_DerivateInterpolants(const TVertex *p_vertex,
	 *		const Vector3f &r_v_direction) const
	 *		- needed to extrapolate vertex coordinates in ContractEdge_UpdateNeighbours
	 *		- returns vertex, containing delta-position (== r_v_direction), delta-normal,
	 *		  delta-color and delta-texcoords in a given direction, n_flag is set to true
	 *		- polygon hash must be available, this function doesn't refresh it
	 *		- in case there is no polygon referencing given vertex, all coordinates of result
	 *		  vertex are set to 0 and n_flag to false
	 */
	TVertex t_DerivateInterpolants(const TVertex *p_vertex,
		const Vector3f &r_v_direction) const;
};

#endif //__GEOMETRY_INCLUDED
