/*
								+---------------------------------+
								|                                 |
								|      ***   OpenGlDrv   ***      |
								|                                 |
								|  Copyright   -tHE SWINe- 2002  |
								|                                 |
								|          OpenGlDrv.cpp          |
								|                                 |
								+---------------------------------+
*/

#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <math.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <vector>

#include "callstack.h"
#include "OpenGlDrv.h"

/*
 *	passed code revision at 05-16-2006
 *
 *	renamed arguments passed to CGLDriver::Init
 *	all integers storing only true or false values were replaced by bool
 *	erased all open-gl init code from CGLDriver::Init, but glDepthFunc(GL_LEQUAL); (default GL_LESS)
 *
 */

/*
 *								=== CGLDriver ===
 */

/*
 *	CGLDriver::CGLDriver()
 *		- default constructor
 */
CGLDriver::CGLDriver()
	:m_b_status(false)
{
	__FuncGuard("CGLDriver::CGLDriver");
}

/*
 *	CGLDriver::~CGLDriver()
 *		- default destructor
 */
CGLDriver::~CGLDriver()
{
	__FuncGuard("CGLDriver::~CGLDriver");

    if(m_b_status)
        Terminate();
}

/*
 *	CGLDriver::Init(HWND h_wnd, int n_width, int n_height, int n_bpp, bool b_fullscreen)
 *		- create OpenGL context for window h_wnd
 *		- display area should have size n_width per n_height pixels, with bit depth n_bpp
 *		- neither window size nor position nor type is changed
 *		- if b_fullscreen is true, it attempts to set display mode n_width per n_height
 *		  pixels and bit depth n_bpp and to hide cursor
 *		- return true on success otherwise false
 *		- on failure it pops ut message box with error description
 */
bool CGLDriver::Init(HWND h_wnd, int n_width, int n_height, int n_bpp, bool b_fullscreen)
{
	__FuncGuard("CGLDriver::Init");

    PIXELFORMATDESCRIPTOR pixelformat;
    DEVMODE dmScreenSettings;
    GLuint PixelFormat;

	if(m_b_status)
		Terminate();

    if(b_fullscreen) {
        memset(&dmScreenSettings, 0, sizeof(dmScreenSettings));
        dmScreenSettings.dmFields = DM_BITSPERPEL | DM_PELSWIDTH | DM_PELSHEIGHT;
        dmScreenSettings.dmSize = sizeof(dmScreenSettings);
        dmScreenSettings.dmPelsWidth = n_width;
        dmScreenSettings.dmPelsHeight = n_height;
        dmScreenSettings.dmBitsPerPel = n_bpp;
        // nastavi rozliseni (pokud fullscreen)

        if(ChangeDisplaySettings(&dmScreenSettings, 4) != DISP_CHANGE_SUCCESSFUL) {
            if(MessageBox(NULL, "Unable to set fullscreen mode.\n"
               "Use windowed mode instead?","Fullscreen", MB_YESNO | MB_ICONEXCLAMATION) == IDYES)
                m_b_fullscreen = 0;
            else
                return false;
        }
        // zkusi prepnout, pokud to nejde, zepta
        // se a pak skonci nebo zustane v okne

        ShowCursor(FALSE);
    }

    memset(&pixelformat, 0, sizeof(PIXELFORMATDESCRIPTOR));
    pixelformat.nSize = sizeof(PIXELFORMATDESCRIPTOR);
    pixelformat.nVersion = 1;
    pixelformat.dwFlags = PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER;
    pixelformat.dwLayerMask = PFD_MAIN_PLANE;
    pixelformat.iPixelType = PFD_TYPE_RGBA;
    pixelformat.cColorBits = n_bpp;
    pixelformat.cDepthBits = n_bpp;
    pixelformat.cAccumBits = 0;
    pixelformat.cStencilBits = 0;
    // nastavi pixel format descriptor

    if(!(m_h_dc = GetDC(m_h_wnd = h_wnd))) {
        MessageBox(NULL, "Unable to create a GL device context.", "GL_Drv", MB_OK);
        return false;
    }
    if(!(PixelFormat = ChoosePixelFormat(m_h_dc, &pixelformat))) {
        MessageBox(NULL, "Unable to find a suitable pixelformat.", "GL_Drv", MB_OK);
        return false;
    }
    if(!SetPixelFormat(m_h_dc, PixelFormat, &pixelformat)) {
        MessageBox(NULL, "Unable to set the pixelformat.", "GL_Drv", MB_OK);
        return false;
    }
    if(!(m_h_glrc = wglCreateContext(m_h_dc))) {
        MessageBox(NULL, "Unable to create a GL rendering context.", "GL_Drv", MB_OK);
        return false;
    }
    if(!wglMakeCurrent(m_h_dc, m_h_glrc)) {
        MessageBox(NULL, "Unable to activate the GL rendering context.", "GL_Drv", MB_OK);
        return false;
    }
    // vytvori OpenGl kontext

    glViewport(0, 0, n_width, n_height);
    // nastavi nase okno

    m_n_width = n_width;
    m_n_height = n_height;
    m_b_status = true;
    m_b_fullscreen = b_fullscreen;
    // nastavi class

	glDepthFunc(GL_LEQUAL);

    return true;
}

/*
 *	bool CGLDriver::Terminate()
 *		- terminates previously initalized context
 *		- return true if no context was initialized or on succes, otherwise false
 *		- if fulscreen mode was set, sets default fullscreen mode and shows cursor
 *		- on failure it pops ut message box with error description
 */
bool CGLDriver::Terminate()
{
	__FuncGuard("CGLDriver::Terminate");

	if(!m_b_status)
		return true;

    bool b_result = true;

    m_b_status = false;

    if(m_b_fullscreen) {
        ChangeDisplaySettings(NULL, 0);
        ShowCursor(TRUE);
    }
    // pokud bylo fullscreen, vrati zpatky

    if(m_h_glrc) {
        if(!wglMakeCurrent(NULL, NULL)) {
            MessageBox(NULL, "Release of DC and RC failed.", "GL_Drv", MB_OK);
            b_result = false;
        }
        if(!wglDeleteContext(m_h_glrc)) {
            MessageBox(NULL, "Release rendering context failed.", "GL_Drv", MB_OK);
            m_h_glrc = NULL;
            b_result = false;
        }
    }

    if(m_h_dc && !ReleaseDC(m_h_wnd, m_h_dc)) {
		MessageBox(NULL, "Release device context failed.", "GL_Drv", MB_OK);
        m_h_dc = NULL;
        b_result = false;
    }

    return b_result;
}

/*
 *	bool CGLDriver::b_Status()
 *		- returns status of open-gl (false = not initialized, true = running)
 */
bool CGLDriver::b_Status()
{
	return m_b_status;
}

/*
 *	void CGLDriver::Blit()
 *		- commits page flip (call glSwapBuffers())
 *		- doesn't call neither glFlush() nor glFinish()
 */
void CGLDriver::Blit()
{
	__FuncGuard("CGLDriver::Blit");

    /*glFlush();
    glFinish();*/
    SwapBuffers(m_h_dc);
}

/*
 *								=== ~CGLDriver ===
 */
