/*
								+----------------------------------+
								|                                  |
								| ***  Pashik virtual machine  *** |
								|                                  |
								|   Copyright  -tHE SWINe- 2002   |
								|                                  |
								|             Pashik.h             |
								|                                  |
								+----------------------------------+
*/

#ifndef __PASHIK_INCLUDED
#define __PASHIK_INCLUDED

#define __Pashik_Ver 0x0110

enum Token_Id {
	Plus, Minus, Multiply, Divide, Modulo,		   // + - * / %
	bOr, bAnd, bNot,                               // || && !
	Or, And, Not, Xor, ShiftR, ShiftL,             // | & ' ~ << >>
	Power, Sin, Cos, Tan, ASin, ACos, ATan,        // ^ sin() cos() tan() asin() acos() atan()
	Log, Ln, Sqrt,								   // log(), ln(), sqrt()
	Floor, Round, Ceil, Frac,					   // floor(), round(), ceil(), frac()
	Begin, End, BegGroup, EndGroup, EndStmt,       // { } [ ] ;
	Open_Par, Close_Par,                           // ( )
	If, Else, Break, Continue, Exit, _Return_,	   // ...
	_For_, _Do_, _While_, _Until_,				   // ...
	Inc, Dec, AddB, SubB, MulB, DivB, PowB, ModB,  // ++_ --_ += -= *= /= ^= %=
	ShLB, ShRB, AndB, OrB, XorB,                   // <<= >>= &= |= ~=
	Equ, NEqu, Greater, Less, GEqu, LEqu,          // == != > < >= <=
	Ident, FValue, IValue, CValue, SValue, Assign, // <identifiktor> int char float string =
	Comma, Float, Int, Short, Char, String, Void,  // , int char float string void
	Post_Inc, Post_Dec, _Unknown_,				   // _++ _--
	Colon, QMark,							       // : ?
	_Enum_, Struct, Typedef, SizeOf,		       // dal opertory
	Inline, Static, Auto,						   // pamov specifiktory
	_Array_, Pointer,							   // tokeny, souvisejc s datovmi typy
	Arrow, Dot,									   // -> .
	Const, Import, Extern, Switch, Case, _EndOfFile_
};
// nzvy token

#define IsBool(a) ((a) == Int || (a) == Short || \
	(a) == Char || (a) == _Enum_ || (a) == Pointer) // new

#define IsNumericOp(a) ((a) >= Plus && (a) <= Power) // new
#define IsFloatOp(a) ((a) >= Plus && (a) <= Divide) // new

#define IsNumber(a) ((a) >= Float && (a) <= Char) // new
#define IsInteger(a) ((a) >= Int && (a) <= Char) // new
#define IsMultiplication(a) (((a) >= Multiply && (a) <= Modulo) || (a) == Power) // new
#define IsDeclSpec(a) ((a) >= Inline && (a) <= Auto) // ok
#define IsVariable(a) ((a) >= Float && (a) <= Void) // ok
#define IsMathFunction(a) ((a) >= Sin && (a) <= Sqrt) // new ->_
#define IsFunction(a) ((a) >= Sin && (a) <= Frac) //            f_ixme : why not floor .. frac ? - for clarify only
#define IsValue(a) ((a) >= IValue && (a) <= FValue) // f_ixme : why not string ? - string not supported
#define IsValuable(a) (IsValue(a) || IsFunction(a) || (a) == Ident) // ok
#define IsOperator(a) ((a) >= Plus && (a) <= Power) // seems to be ok
#define IsCycle(a) ((a) >= _For_ && (a) <= _Until_) // ok
#define IsJump(a) ((a) >= Break && (a) <= _Return_) // ok, but modified !!!!
#define IsRelation(a) ((a) >= Equ && (a) <= LEqu) // ok
#define IsAssignment(a) (((a) >= AddB && (a) <= XorB) || (a) == Assign) // ok
#define IsPre_Unary(a) ((a) == Plus || (a) == Minus || (a) == Inc ||\
						(a) == Dec || (a) == And || (a) == Not || (a) == bNot) // ok
#define IsPost_Unary(a) ((a) == Post_Inc || (a) == Post_Dec) // ok
#define IsUnaryOp(a) (IsPre_Unary(a) || IsPost_Unary(a)) // new
#define IsBooleanOp(a) ((a) >= bOr && (a) <= bNot) // ok
// makra pro zjiovn typ token

struct Token {
	Token_Id t_id;
	//
	int n_line;
	//
	float f_value;
	long n_value;
	char *p_s_value;
};
// token

struct TDataType;

struct TSimpleDataType {
	int n_datatype_id; // id odpovdajcho datatypu (v tabulce)
	Token_Id n_type;
	int n_struct_size; // velikost typu validn pro st_Struct / st_Pointer (velikost podstruktury)
};

struct TOpCode {
	int n_address; // debug
	int n_code;
	int n_data[3];
	TSimpleDataType t_operand_type[3];
};
// instrukce pro VM

struct TFuncHead {
	char *p_s_name;

	int b_extern;
	int b_import;

	TSimpleDataType t_return_type;
	int n_param_num;
	struct TParam {
		TSimpleDataType t_param_type;
		int n_param_address; // u import pouvna pro ukldn parametr do pole
	} *p_param;				 // k uloen do zsobnku, u export k zadvn parametr
};

struct TProgram {
	int n_id;

	struct TMemoryContainer {
		unsigned char *p_constant_memory;
		int n_constant_memory_size;
		int n_memory_size; // (including const memory)
		//
		int n_address_item_num; // poet adresnch konstant v pamti
		int *p_address_item; // a jejich adresy
		//
		int n_stack_size;
	} t_memory;

	struct TCodeContainer {
		int n_length;
		TOpCode *p_code;
	} t_code;

	struct TImportContainer {
		int n_import_num;
		struct TImport {
			int n_id;
			TFuncHead t_function;
			void *p_address; // adresa kdu funkce (__stdcall)
		} *p_import;
	} t_imports;

	struct TExportContainer {
		int n_export_num;
		struct TExport {
			int n_id;
			TFuncHead t_function;
			int n_entry_address;
		} *p_export;
	} t_exports;

	struct TDataTypeContainer {
		int n_type_num;
		TDataType **p_type;
	} t_data_types;

	int Write(FILE *p_fw);
	int Read(FILE *p_fw);

	void Free();
};

void Warning(const char *s_err, int n_line);
void Error(const char *s_err, int n_line);
// vype chybu a pamatuje si poet (v pashik.cpp, global)

void Pashik_Silence();
// uml chybov vstup

enum {
	Pash_Ok,
	Pash_Error,
	Pash_NoFile,
	Pash_NoMem,
	Pash_Syntax,
	Pash_Parser,
	Pash_CodeGen,

	Pash_NumErrors
};

short Pashik_GetDllVersion();

class CSymbolTable;
class CMemManager;

class CPashik {
protected:
	void (*m_p_pas_err_routine)(char *s_err);
	int m_n_last_error;

public:
	CPashik();
	void Set_ErrorRoutine(void (*err_routine)(char *s_err));

	int Compile(const char *p_s_buffer, int n_length, TProgram *p_program);
	int Compile(const char *p_s_file, TProgram *p_program);

	int n_GetLastError();

protected:
	int Design_Memory(TProgram *p_program,
		CSymbolTable *p_symbol_table, CMemManager *p_memory_manager);
	int Copy_DataTypes(TProgram *p_program, CSymbolTable *p_symbol_table);
	TDataType *p_FindType(TProgram *p_program, int n_type_id);
};

// init

enum ECompileMode {
	CM_ChkErrors,
	CM_Compile
};

int Set_StackSize(int n_size);
//
void Set_MemoryOut(int b_active, char *p_s_filename);
void Set_ParserOut(int b_active, char *p_s_filename);
void Set_SymtabOut(int b_active, char *p_s_filename);
//
void Set_CompileMode(ECompileMode e_mode);

#endif
